/*
fluxy. A Realtime 2d Path Tracer
---
Copyright (C) 2008  Sebastian Mach (*1983)
seb@greenhybrid.net | http://greenhybrid.net
---
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <float.h>
#include <limits.h>

LineList::LineList()
: lines(NULL), numLines(0), numLuminaires(0) {
}

void LineList::beginSceneDefinition(){
    numLuminaires = 0;
}

void LineList::endSceneDefinition(){
    if( numLines != compile_lines.size() ){ // have to reallocate?
        if( NULL != lines )
            delete [] lines;
        lines = new FastLine[compile_lines.size()];
    }
    numLines = compile_lines.size();
    for( unsigned int u=0; u<numLines; ++u ){
        //lines[u] = compile_lines[u];
        lines[u].isLuminaire = u<numLuminaires;

        lines[u].x = compile_lines[u].begin.x;
        lines[u].y = compile_lines[u].begin.y;

        const Vector l = compile_lines[u].end - compile_lines[u].begin;
        const Vector d = l.normal();
        lines[u].d_x = l.x;
        lines[u].d_y = l.y;
        lines[u].n_x[0] = -d.y;
        lines[u].n_y[0] = d.x;
        lines[u].n_x[1] = d.y;
        lines[u].n_y[1] = -d.x;
        //lines[u].length = l.length();

        lines[u].material = compile_lines[u].material;
    }
    compile_lines.clear();
}

void LineList::addLine( Line line ){
    if( line.material.isLuminaire ){
        compile_lines.insert( compile_lines.begin(), line );
        ++numLuminaires;
    }else{
        compile_lines.push_back( line );
    }
}


static float intersectRayRay(
    float Ax, float Ay, float Adx, float Ady,
    float Bx, float By, float Bdx, float Bdy
){
    // TODO: optimize
#if 0
    const float &x1 = Ax;
    const float  x2 = Ax+Adx;
    const float &x3 = Bx;
    const float  x4 = Bx+Bdx;
    const float &y1 = Ay;
    const float  y2 = Ay+Ady;
    const float &y3 = By;
    const float  y4 = By+Bdy;
    const float den = 1.0 / ( (y4-y3)*(x2-x1)-(x4-x3)*(y2-y1) );
    const float ua = ( (x4-x3)*(y1-y3)-(y4-y3)*(x1-x3) ) * den;
    const float ub = ( (x2-x1)*(y1-y3)-(y2-y1)*(x1-x3) ) * den;
    if( ub>=0.0f && ub<=1.0f && ua>=0.0f )
        return ua;
    return -1.0f;
#else
    const float den = 1.0 / ( (Bdy)*(Adx)-(Bdx)*(Ady) );
    const float alpha = Ay-By;
    const float bravo = Ax-Bx;
    const float ua = ( Bdx*alpha-Bdy*bravo ) * den;
    const float ub = ( Adx*alpha-Ady*bravo ) * den;
    const float ret[2] = { -1.0f, ua };
    return ret[(ub>=0.0f) & (ub<=1.0f) & (ua>=0.0f)];
#endif
}

Intersection LineList::intersect( float x, float y, float d_x, float d_y ) const{
#if 0
    unsigned int nearest = UINT_MAX;
    float nearestDistance = FLT_MAX;

    for( unsigned int u=0; u<numLines; ++u ){
        const float tmp = intersectRayRay( x, y, d_x, d_y, lines[u].x, lines[u].y, lines[u].d_x, lines[u].d_y );
        if( tmp > 0.0f && tmp < nearestDistance ){
            nearestDistance = tmp;
            nearest = u;
        }
    }

    static const Intersection noIntersection( Material( ColorFRGB(0.0f, 0.0f, 0.0f) ), Vector(), -1.0f );
    //const Intersection[2] =
    if( nearest == UINT_MAX ){
        return noIntersection;
    }else{
        const int n = (-lines[nearest].d_y*d_x + lines[nearest].d_x*d_y)>0.0f;
        const float nx[2] = { -lines[nearest].d_y,  lines[nearest].d_y };
        const float ny[2] = {  lines[nearest].d_x, -lines[nearest].d_x };
        return Intersection( lines[nearest].material, Vector( nx[n], ny[n] ).normal(), nearestDistance );
    }
#else
    unsigned int nearest[2] = { UINT_MAX, UINT_MAX };
    float nearestDistance[2] = { FLT_MAX, FLT_MAX };

    for( unsigned int u=0; u<numLines; ++u ){
        const float tmp = intersectRayRay( x, y, d_x, d_y, lines[u].x, lines[u].y, lines[u].d_x, lines[u].d_y );
        const int n = (tmp<0.0f) | (tmp>*nearestDistance);
        nearestDistance[n] = tmp;
        nearest[n] = u;
    }

    const int n = (-lines[*nearest].d_y*d_x + lines[*nearest].d_x*d_y)>0.0f;
    static Intersection k[2] = {
        Intersection( Material( ColorFRGB(0.0f, 0.0f, 0.0f) ), Vector(), -1.0f ),
        Intersection( Material( ColorFRGB(0.0f, 0.0f, 0.0f) ), Vector(), -1.0f )
    };
    // meh, voodoo (but yields to a total of 2 Intersection::Intersection calls, yay)
    const_cast<Material&>(k[0].material) = lines[*nearest].material;
    const_cast<Vector&>(k[0].normal)     = Vector( lines[*nearest].n_x[n], lines[*nearest].n_y[n] );
    const_cast<float&>(k[0].distance)    = *nearestDistance;
    return k[*nearest == UINT_MAX];
#endif
}

