/*
fluxy. A Realtime 2d Path Tracer
---
Copyright (C) 2008  Sebastian Mach (*1983)
seb@greenhybrid.net | http://greenhybrid.net
---
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef VECTOR_H
#define VECTOR_H

#include <math.h>

struct Vector{
    union{
        struct{
            float x,y;
        };
        float m[2];
    };
    Vector() : x(0.0f), y(0.0f) {
    }
    Vector( float x, float y ) : x(x), y(y) {
    }

    float length() const {
        return sqrt( x*x + y*y );
    }
    float length_sq() const {
        return x*x + y*y;
    }

    Vector normal() const {
        const float l = 1.0 / length();
        return Vector( x*l, y*l );
    }

    /*friend inline Vector operator + ( const Vector& a, const Vector& b );
    friend inline Vector operator - ( const Vector& a, const Vector& b );
    friend inline Vector operator * ( const Vector& a, float b );*/
};

static inline Vector operator + ( const Vector& a, const Vector& b ) {
    return Vector( a.x+b.x, a.y+b.y );
}

static inline Vector operator - ( const Vector& a, const Vector& b ) {
    return Vector( a.x-b.x, a.y-b.y );
}

static inline Vector operator * ( const Vector& a, float b ) {
    return Vector( a.x*b, a.y*b );
}

static inline Vector operator / ( const Vector& a, float b ) {
    return Vector( a.x/b, a.y/b );
}


#endif // VECTOR_H
